/* 
 * File:   motorvariables.h
 * Author: dsatya
 *
 * Created on 14 May, 2017, 6:30 PM
 */

#ifndef MOTORVARIABLES_H
#define	MOTORVARIABLES_H

#ifdef	__cplusplus
extern "C" {
#endif
typedef struct axis_tag
{
    struct abcaxis_tag
    {
        double32_t g_a_axis_d32;
        double32_t g_b_axis_d32;
        double32_t g_c_axis_d32;
    }abcaxis;
    struct alpha_beta_axis_tag
    {
        double32_t g_alpha_axis_d32;
        double32_t g_beta_axis_d32;
    }alpha_beta_axis;
    struct d_q_axis_tag
    {
        double32_t g_d_axis_d32;
        double32_t g_q_axis_d32;
    }d_q_axis;
}axis_T;

typedef struct current_tag
{
    axis_T g_current_measured_s;
    axis_T g_current_reference_s;
}current_T;

typedef struct voltage_tag
{
    axis_T g_voltage_measured_s;
    axis_T g_voltage_reference_s;
}voltage_T;

typedef struct error_tag
{
    double32_t g_speed_error_d32s;
    double32_t g_current_error_d32s;
}error_T;

typedef struct reference_parameters_tag
{
    double32_t g_ref_torque_d32;
    double32_t g_ref_speed_d32;
    double32_t g_ref_position_d32;
}motor_ref_parametres_T;

typedef struct actual_parameters_tag
{
    double32_t g_measured_torque_d32;
    double32_t g_measured_speed_d32;
    sint16_t g_measured_position_theta_s16;
}motor_measured_parametres_T;

typedef struct
{		
uint16_t l_receive_id_u16;
uchar8_t l_receive_dlc_u8;
uchar8_t l_can_input_u8A[8];
}CANRXID;

extern current_T g_current_s;
extern voltage_T g_voltage_s;
extern motor_ref_parametres_T g_motor_ref_parametres_s;
extern motor_measured_parametres_T g_motor_measured_parameters_s;

#define PHASE_A_FRE         (3499u)  /* 20KHz*/
#define PHASE_B_FRE         (3499u)  /* 20KHz*/
#define PHASE_C_FRE         (3499u)  /* 20KHz */
#define DUTY_PHASE_A        (50u)
#define DUTY_PHASE_B        (50u)
#define DUTY_PHASE_C        (50u)

#ifdef	__cplusplus
}
#endif

#endif	/* MOTORVARIABLES_H */

