
/*******************************************************************************
  Company:    Comstar Automotive Technologies.
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	

  File Name:     diagnosis.c

  Summary:  fault monitoring

Description: 

 This is a daignosis file, which monitors the faults in the ISG system.
	
 Created by  Vijay S 
 Date      : 10th Nov 2017
 Version   : 1.0 [BASE VERSION -Platform]
 
 
 *******************************************************************************/

/******************************************************************************
 *****************************************************************************
 Section: Included Files
 *****************************************************************************
 *****************************************************************************/

#include <xc.h>
#include <string.h>
#include <types.h>


/*****************************************************************************
 *****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************
 *****************************************************************************/


#define BOARD_TEMP   (3u)
#define MINIMUM_PHASE_CURRENT (10u)
#define MAXIMUM_PHASE_CURRENT (40u)
#define BVALUE (3988.0f)
#define T0     (298.0f)





#define PHASE_3_CURRENT (19u)
#define PHASE_2_CURRENT (30u)

#define PHASE_1_CURRENT (55u)

#define COEFF (-(3988.0f/298.0f))


/***************************************************************************
  Section: Function  prototype
****************************************************************************/
extern uint16_t ADC_Read_fun(uchar8_t a_channel_u8);
/*****************************************************************************
  Section: File Scope Variables, Functions, Initializations
 *****************************************************************************/
uint16_t ADCValue1=0;
uchar8_t g_phase_Current_Sign_u8=0;

uchar8_t g_phase_rds_value_u8[37]={46,48,50,52,53,55,56,58,60,62,63,65,66,68,70,73,76,78,
                          80,83,86,90,92,93,96,100,103,106,110,113,116,120,123,126,130,133,136};

float32_t g_boardtemperature_d32=0.0f;
float32_t g_phase_1_current_d32=0.0f;
float32_t g_phase_2_current_d32=0.0f;
float32_t g_phase_3_current_d32=0.0f;
float32_t g_adc_phase_1_current_d32=0.0f;
float32_t g_adc_phase_2_current_d32=0.0f;
float32_t g_adc_phase_3_current_d32=0.0f;


float32_t g_adc_phase1_ampl_volt_d32=0.0f;
float32_t g_adc_phase2_ampl_volt_d32=0.0f;
float32_t g_adc_phase3_ampl_volt_d32=0.0f;

uint16_t g_adc_phase1_count_u16=0;
uint16_t g_adc_phase2_count_u16=0;
uint16_t g_adc_phase3_count_u16=0;

/*****************************************************************************/
/*********************************************************************
***
*** Function Name: 
***
*** Description:
***
*** Parameter Description:
***       
*** Return type: 
********************************************************************/ 
void Phase1CurrentMonitoring(void)
{                   
 float32_t l_phase_1_current_d32=0;
 uint16_t l_rdsvalue_u16=0;
 
g_adc_phase1_count_u16= ADC_Read_fun(PHASE_1_CURRENT); 
g_adc_phase_1_current_d32=(float32_t)g_adc_phase1_count_u16;

g_adc_phase_1_current_d32= (g_adc_phase_1_current_d32 * 0.00122);
 
 if(g_adc_phase_1_current_d32 > 2.5)
 {
 g_adc_phase1_ampl_volt_d32=((g_adc_phase_1_current_d32 -2.5) * 100 )/ (0.12);
 g_phase_Current_Sign_u8 |= 0x02; 
 }
 else if(g_adc_phase_1_current_d32 < 2.5)
 {
      g_adc_phase1_ampl_volt_d32=((2.5 - g_adc_phase_1_current_d32) * (100))/ (0.12);
     g_phase_Current_Sign_u8 |= 0x01;
 }
 
 
 
}

/*********************************************************************
***
*** Function Name: 
***
*** Description:
***
*** Parameter Description:
***       
*** Return type: 
********************************************************************/
void Phase2CurrentMonitoring(void)
{ 
  float32_t l_phase_2_current_d32=0;
  
  g_adc_phase2_count_u16=ADC_Read_fun(PHASE_2_CURRENT);
g_adc_phase_2_current_d32=(float32_t)g_adc_phase2_count_u16;  
 g_adc_phase_2_current_d32=(g_adc_phase_2_current_d32*0.00122);
  
  if(g_adc_phase_2_current_d32 > 2.5)
 {
 g_adc_phase2_ampl_volt_d32=((g_adc_phase_2_current_d32 -2.5)* 100)/ (0.12);
 g_phase_Current_Sign_u8 |= 0x08; 
 }
 else if(g_adc_phase_2_current_d32 < 2.5)
 {
      g_adc_phase2_ampl_volt_d32=((2.5 - g_adc_phase_2_current_d32) * 100)/ (0.12);
     g_phase_Current_Sign_u8 |= 0x04;
 }        
          
          
  
}

/*********************************************************************
***
*** Function Name: 
***
*** Description:
***
*** Parameter Description:
***       
*** Return type: 
********************************************************************/
void Phase3CurrentMonitoring(void)
{
  float32_t l_phase_3_current_d32=0;
  
  g_adc_phase3_count_u16=ADC_Read_fun(PHASE_3_CURRENT);
  
  g_adc_phase_3_current_d32=(float32_t)g_adc_phase3_count_u16;
  g_adc_phase_3_current_d32=(g_adc_phase_3_current_d32*0.00122);
  
  if(g_adc_phase_3_current_d32 > 2.5)
 {
      g_adc_phase3_ampl_volt_d32=((g_adc_phase_3_current_d32 -2.5)* 100)/ (0.12);
      g_phase_Current_Sign_u8 |= 0x20; 
  
  }
 else if(g_adc_phase_3_current_d32 < 2.5)
 {
      g_adc_phase3_ampl_volt_d32=((2.5 - g_adc_phase_3_current_d32) * 100)/ (0.12);
      g_phase_Current_Sign_u8 |= 0x10;
 }
  
  
   
}


void IsgmSignalsMonitoring(void)
{ 
  //Phase1CurrentMonitoring();
  //Phase2CurrentMonitoring();
  //Phase3CurrentMonitoring();

}

uint16_t ADC_Read_fun(uchar8_t a_channel_u8)
{
  AD1CHS0 = a_channel_u8;
  AD1CON1bits.SAMP = 1; // Start sampling
  Delay_us(30);  // Wait for sampling time (10 us)
  AD1CON1bits.SAMP = 0;  // Start the conversion
  while (!AD1CON1bits.DONE);  // Wait for the conversion to complete
  ADCValue1 = ADC1BUF0;  // Read the ADC conversion result
  return (ADCValue1);
}
