/*******************************************************************************
 focalgorithm.c

  Company:    Comstar Automotive Technologies.
	
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	
	

File Name:     focalgorithm.c

Summary:     This is the control algorihm for sensored foc.

Description: 

 This program is the control of the foc this is called from pwm interrupt every 60us. 
	
 Created by 
 Date      : 14th May 2017
 Version   : 1.0 [BASE VERSION]
 
*******************************************************************************/

/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"

/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/
#define IQRINIT_NOMINAL_CURRENT   (10u)

/****************************************************************************
 * Section: File Scope Variables, Functions, Initializations
 * 
 *****************************************************************************/
extern const int sinelut[250];
extern const int coslut[251];
extern uint16_t g_markpulse_a_u16;
extern uint16_t g_markpulse_b_u16;
extern uchar8_t g_markpulse_a_flag_u8;
extern uchar8_t g_markpulse_b_flag_u8;
extern uchar8_t g_clockwisedirection_u8;
extern uchar8_t g_anticlockwise_direction_u8;
extern uint16_t g_encoderpulses_u16;
extern uchar8_t g_markflag_u8;
struct g_angleinfo_tag_s
{
    sint16_t g_sinevalue_s16;
    sint16_t g_cosvalue_s16;
}g_angleinfo_s;

/****************************************************************************
  Section: Function Declarations, extern declarations
 *****************************************************************************/
void CalculatePositionAndDirection(void);
void CalculatePosition(void);
void FocAlgorithm(void);
void InitializeRotorToKnownPosition(void);
void CalculatePositionAndDirection(void);
extern void ClarkeTransform(current_T *a_ptr_s);
extern void ParkTransform(current_T *a_current_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s);
extern void InverseParkTransform(voltage_T *a_voltage_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s);
extern void InverseClarkeTransform(voltage_T *a_voltage_ptr_s);
extern double32_t PIregulator_Q(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);
extern double32_t PIregulator_D(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);
extern double32_t PIregulator_Speed(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);

extern void SvPwm(void);
void CalculatePosition_test(void);

void FocAlgorithm(void)
{
    /**/
 motor_ref_parametres_T *l_ref_ptr_s = &g_motor_ref_parametres_s;
 /* if any of the values are not with default values then it 
  can be assumed that the command has been received from CAN to go to either of modes 
  */
 if(l_ref_ptr_s->g_ref_position_d32 != -1.0 || l_ref_ptr_s->g_ref_speed_d32 != -1.0 || l_ref_ptr_s->g_ref_torque_d32 != -1.0)
 {
     
 }
 else
 {
  InitializeRotorToKnownPosition();
 }
 CalculatePositionAndDirection();
 ClarkeTransform(&g_current_s); /* Apply clarke transform on the a,b,c currents*/
 ParkTransform(&g_current_s,&g_motor_measured_parameters_s);
 if(l_ref_ptr_s->g_ref_speed_d32 != -1.0) /* Speed command received */
 {
  /* Execute the speed regulator */
  g_voltage_s.g_voltage_measured_s.d_q_axis.g_q_axis_d32 = PIregulator_Speed(&g_motor_ref_parametres_s.g_ref_speed_d32,&g_motor_measured_parameters_s.g_measured_speed_d32);
 }
 else
 {
  g_voltage_s.g_voltage_measured_s.d_q_axis.g_q_axis_d32 = PIregulator_Q(&g_current_s.g_current_reference_s.d_q_axis.g_q_axis_d32,&g_current_s.g_current_measured_s.d_q_axis.g_q_axis_d32); /* */
 }
 g_voltage_s.g_voltage_measured_s.d_q_axis.g_d_axis_d32 = PIregulator_D(&g_current_s.g_current_reference_s.d_q_axis.g_d_axis_d32,&g_current_s.g_current_measured_s.d_q_axis.g_d_axis_d32);
 InverseParkTransform(&g_voltage_s,&g_motor_measured_parameters_s);
 InverseClarkeTransform(&g_voltage_s);
 SvPwm();
}

void InitializeRotorToKnownPosition(void)
{
 g_current_s.g_current_reference_s.d_q_axis.g_q_axis_d32 = IQRINIT_NOMINAL_CURRENT;   
 g_current_s.g_current_reference_s.d_q_axis.g_d_axis_d32 = 0;
 g_motor_measured_parameters_s.g_measured_position_theta_s16 = 0;
}


void CalculatePositionAndDirection(void)
{
  if(TRUE == g_markflag_u8)
  {
    if ((TRUE == g_markpulse_a_flag_u8) && (TRUE == g_markpulse_b_flag_u8))
    {
        g_markpulse_a_flag_u8 = FALSE;
        g_markpulse_b_flag_u8 = FALSE;
        
        if (g_markpulse_a_u16 > g_markpulse_b_u16)
        {
           g_clockwisedirection_u8 = TRUE; 
        }
        else
        {
           g_anticlockwise_direction_u8 = TRUE; 
        }
     g_encoderpulses_u16++;
     CalculatePosition();
    } 
  }
}



void CalculatePosition(void)
{
    if(g_encoderpulses_u16 > 0u && g_encoderpulses_u16 <= 250u)
    {
      g_angleinfo_s.g_sinevalue_s16 = sinelut[g_encoderpulses_u16];
      g_angleinfo_s.g_cosvalue_s16 = coslut[g_encoderpulses_u16];
       
    }
    else if(g_encoderpulses_u16 > 250 && g_encoderpulses_u16 <= 500)
    {
      g_angleinfo_s.g_sinevalue_s16 = sinelut[500 - g_encoderpulses_u16];
      g_angleinfo_s.g_cosvalue_s16 = -coslut[500 - g_encoderpulses_u16]; /*90 to 180 degrees negative */
    }
    else if(g_encoderpulses_u16 > 500u && g_encoderpulses_u16 <= 750u)
    {
      g_angleinfo_s.g_sinevalue_s16 = -sinelut[g_encoderpulses_u16 - 750u];
      g_angleinfo_s.g_cosvalue_s16 = -coslut[g_encoderpulses_u16 - 750u]; /*180 to 270 degrees negative */
    }
    else if(g_encoderpulses_u16 > 750u && g_encoderpulses_u16 < 1000u)
    {
      g_angleinfo_s.g_sinevalue_s16 = -sinelut[1000 - g_encoderpulses_u16];
      g_angleinfo_s.g_cosvalue_s16 = coslut[1000 - g_encoderpulses_u16]; /*270 to 360 degrees negative */
    }
   
   if(TRUE == g_anticlockwise_direction_u8 )
   {
     g_motor_measured_parameters_s.g_measured_position_theta_s16 = -g_motor_measured_parameters_s.g_measured_position_theta_s16;  
   }
   else if(TRUE == g_clockwisedirection_u8 )
   {
     g_motor_measured_parameters_s.g_measured_position_theta_s16 = g_motor_measured_parameters_s.g_measured_position_theta_s16;
   }
}


