#include <xc.h>
#include "types.h"
#include "motorvariables.h"

_FOSCSEL(FNOSC_FRC & IESO_OFF);
_FOSC(FCKSM_CSECMD & OSCIOFNC_OFF & POSCMD_XT);
_FWDT(FWDTEN_OFF)
_FICD(ICS_PGD2)

#pragma config DMTEN = 0
        
 #pragma config PWMLOCK = OFF            // PWM Lock Enable Bit (PWM registers may be written without key sequence)
 #pragma config BOREN0 = OFF             // Brown Out Reset Detection Bit (BOR is Disabled)        

        
        
/*****************************************************************************
 *****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************
 *****************************************************************************/
#define NULL             (void *)0u
#define TORQUE_COMMAND   (0x01u)
#define SPEED_COMMAND    (0x02u)
#define STOP_COMMAND     (0x03u)
#define TORQUE_MSGID     (0x103u)
#define SPEED_MSGID      (0x104u)
#define STOP_MSGID       (0x105u)
#define MHZ_20           (18u)
#define MHZ_40           (38u)
#define MHZ_100          (82u)
#define MHZ_60           (58u)
#define MAXPULSES        (1000u)
        
 extern uint16_t Phase;
 extern const int PhaseValues[6];


extern CANRXID g_canRxMessage;
extern uchar8_t g_datarxed_u8;
extern uchar8_t g_hallAstatus;
extern uchar8_t g_hallBstatus;
extern uchar8_t g_hallCstatus;
extern uint16_t g_period_u16;

uchar8_t g_sector_array[8] = {-1, 4, 0, 5, 2, 3, 1,-1};     /* Actual edge */
//uchar8_t g_sector_array[8] = {-1, 5, 1, 0, 3, 2, 4,-1};     /* Actual edge */
double32_t g_RPM = 0.0;
uchar8_t g_sector = 0;


//---------------------------------------------------------------------
// These are the definitions for various angles used in the SVM 
// routine.  A 16-bit unsigned value is used as the angle variable.
// The SVM algorithm determines the 60 degree sector

extern uchar8_t g_hall_a_greterthan_hall_b_u8;
extern uchar8_t g_hall_b_greterthan_hall_c_u8;
extern uchar8_t g_hall_c_greterthan_hall_a_u8;
/****************************************************************************
  Section: Function Declarations, extern declarations
*****************************************************************************/

extern void PwmInitialization(void);
extern void AdcInitialization(void);
extern void InputCaptureInitialization(void);
extern void Init_Timer1(void);
extern void Init_Timer3(void);
extern void AdcInit(void);
extern void ComparatorInit(void);
extern void ComparatorOut(void);
void Init_HWport(void);
void PWM_control(void);

int main(void)
{
   
   PLLFBD=68;              // M=48  /* 68 for 70MHz, 28 for 20MHz */
   CLKDIVbits.PLLPOST=0;   // N2=2
   CLKDIVbits.PLLPRE=0;    // N1=2 
   
   // Initiate Clock Switch to Primary Oscillator with PLL (NOSC=0b011)
__builtin_write_OSCCONH(0x03);
__builtin_write_OSCCONL(OSCCON | 0x01);
// Wait for Clock switch to occur
while (OSCCONbits.COSC!= 0b011);
// Wait for PLL to locke
while (OSCCONbits.LOCK!= 1);
DMTCONbits.ON = 0; // AdcInit();
   

    Init_HWport();
   //TogglePortInit();
   Init_Timer1();
   Init_Timer3();
   //PwmInitialization();
   PwmInitUpdated();
  
   inputcapture1_init();
   ComparatorInit();
  // PortHigh();
    initAdc1();
    Can_Init();
   while(TRUE)
   {
       RunMotor();
        while(1)
        {
           // SectorMain();
        }
   }
  return 0;
}

 void TogglePortInit(void)
 {
     _TRISD6 = 0;
     _RD6 = 0;
     
     ANSELGbits.ANSG6 = 0;
     _TRISG6 = 0;
     _RG6 = 1;
 }
 
 void TogglePort(void)
 {
     static uchar8_t l_debug_u8=0;
     _RD6 = l_debug_u8;
     l_debug_u8 = ~l_debug_u8;
     
    //_RD6 = PORTDbits.RD8;
 }
 
 void PortHigh(void)
 {
   _RD6 = 1; 
 }
 
 void PortLow(void)
 {
   _RD6 = 0; 
 }
 
 void Init_HWport(void)
 {
   MicroControls();
 }
 
extern uint16_t g_pastcapture_u16;
extern uint16_t g_actualcapture_u16;
#define MAXPERIOD        8191u
#define PHASE_INC_CALC 448000UL
extern signed int PhaseInc;
extern unsigned int LastSector;
#define CW	0		// Counter Clock Wise direction
#define CCW	1		// Clock Wise direction

extern int PhaseOffsetCW;
extern int PhaseOffsetCCW;
extern unsigned char Current_Direction;	// Current mechanical motor direction of 
                                    // rotation Calculated in halls interrupts
extern unsigned char Required_Direction;	// Required mechanical motor direction of 
                                    // rotation, will have the same sign as the
									// ControlOutput variable from the Speed 
                                    // Controller
extern int ControlOutput;
extern int RefSpeed;
void RunMotor(void)
{
	// init variables
    uchar8_t HallValue=0;
	
	TMR1 = 0;			// Reset timer 1 for speed control
	TMR3 = 0;			// Reset timer 3 for speed measurement
	g_actualcapture_u16 = MAXPERIOD; 	// Initialize captures for minimum speed 
								//(60 RPMs)
	g_pastcapture_u16 = 0;
	// Initialize direction with required direction
	// Remember that ADC is not stopped.
    g_hall_a_greterthan_hall_b_u8 = CMSTATbits.C2OUT;
    g_hall_c_greterthan_hall_a_u8 = CMSTATbits.C3OUT;
    g_hall_b_greterthan_hall_c_u8 = CMSTATbits.C1OUT;
    
	HallValue = g_hall_a_greterthan_hall_b_u8+2*g_hall_b_greterthan_hall_c_u8+4*g_hall_c_greterthan_hall_a_u8;// Read halls	
	LastSector = g_sector = g_sector_array[HallValue];	// Initialize Sector 
                                                    // variable

	// RefSpeed's sign will determine if the motor should be run at CW 
    // (+RefSpeed) or CCW (-RefSpeed) ONLY at start up, since when the motor 
    // has started, the required direction will be set by the control output
    // variable to be able to operate in the four quadrants
	if (RefSpeed < 0)
	{
		ControlOutput = 0;	// Initial output voltage
		Current_Direction = Required_Direction = CCW;
		Phase = PhaseValues[(g_sector + 3) % 6]+PhaseOffsetCCW;
	}
	else
	{
		ControlOutput = 32766;	// Initial output voltage
		Current_Direction = Required_Direction = CW;
		Phase = PhaseValues[(g_sector) % 6] + PhaseOffsetCW;
	}
	//MotorStalledCounter = 0;	// Reset motor stalled protection counter
	// Set initial Phase increment with minimum value. This will change if a 
    // costing operation is required by the application
	PhaseInc = __builtin_divud(PHASE_INC_CALC, MAXPERIOD);	
	// Clear all interrupts flags
	IFS0bits.T1IF = 0;		// Clear timer 1 flag
	IFS0bits.IC1IF = 0;		// Clear interrupt flag
	IFS0bits.IC2IF = 0;		// Clear interrupt flag
	IFS2bits.IC3IF = 0;		// Clear interrupt flag
	IFS5bits.PWM1IF = 0;	// Clear interrupt flag
	// enable all interrupts
	__asm__ volatile ("DISI #0x3FFF");
	IEC0bits.T1IE = 1;		// Enable interrupts for timer 1
	IEC0bits.IC1IE = 1;		// Enable interrupts on IC1
	IEC0bits.IC2IE = 1;		// Enable interrupts on IC2
	IEC2bits.IC3IE = 1;		// Enable interrupts on IC3
	IEC5bits.PWM1IE = 1;	// Enable PWM interrupts
    DISICNT = 0;
	//Flags.MotorRunning = 1;	// Indicate that the motor is running
	return;
}
 
 void SectorMain(void)
 {
     uchar8_t l_hall_u8=0;
    g_hall_a_greterthan_hall_b_u8 = CMSTATbits.C2OUT;
    g_hall_c_greterthan_hall_a_u8 = CMSTATbits.C3OUT;
    g_hall_b_greterthan_hall_c_u8 = CMSTATbits.C1OUT;
    
	l_hall_u8 = g_hall_a_greterthan_hall_b_u8+2*g_hall_b_greterthan_hall_c_u8+4*g_hall_c_greterthan_hall_a_u8;// Read halls	
    g_sector = g_sector_array[l_hall_u8];	// Initialize Sector 
    
    
 }