/*******************************************************************************
 regulator.c

  Company:    Comstar Automotive Technologies.
	
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	
	

File Name:     regulator.c

Summary:     This is the regulator file for minimizing the error condition 

Description: 

 This file uses the proportional integral loop to minimize the error between the 
 * measured value and the reference value. 
	
 Created by 
 Date      : 14th May 2017
 Version   : 1.0 [BASE VERSION]
 
*******************************************************************************/

/****************************************************************************** 
 ******************************************************************************
 Section: Included Files
 ******************************************************************************
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"
/*****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************/
#define MAX_VOLTAGE   (60u)
#define MIN_VOLTAGE   (2u)
#define KI            (1.2f)
#define KPI           (0.2f)

/****************************************************************************
  Section: File Scope Variables, Functions, Initializations
 *****************************************************************************/
double32_t g_xi_q_d32=0.0;
double32_t g_xi_d_d32=0.0;
double32_t g_xi_speed_d32=0.0;

/****************************************************************************
  Section: Function Declarations, extern declarations
 *****************************************************************************/
double32_t PIregulator_Q(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);
double32_t PIregulator_D(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);
double32_t PIregulator_Speed(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32);

double32_t PIregulator_Q(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32)
{
   double32_t l_refvalue_d32=0.0;
   double32_t l_measure_d32=0.0;
   double32_t l_error_d32=0.0;
   double32_t l_error_limit_d32=0.0;
   double32_t l_output_voltage_d32=0.0;
   double32_t l_output_limit_voltage_d32=0.0;
   
   l_refvalue_d32 = *a_refvalue_d32;
   l_measure_d32 = *a_measuredvalue_d32;
   l_error_d32 = -l_measure_d32 + l_refvalue_d32;
   l_output_voltage_d32 = g_xi_q_d32 + KPI * l_error_d32;
   l_output_limit_voltage_d32 = l_output_voltage_d32;
   if(l_output_voltage_d32 > MAX_VOLTAGE)
   {
     l_output_limit_voltage_d32 =  MAX_VOLTAGE; 
   }
   else if(l_output_voltage_d32 < MIN_VOLTAGE)
   {
   //  l_output_limit_voltage_d32 = MIN_VOLTAGE;  
   }
   l_error_limit_d32 = l_output_voltage_d32 - l_output_limit_voltage_d32;
   g_xi_q_d32 = g_xi_q_d32 + (KI * l_error_d32) + (l_error_limit_d32);
   return l_error_d32;       
}

double32_t PIregulator_D(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32)
{
   double32_t l_refvalue_d32=0.0;
   double32_t l_measure_d32=0.0;
   double32_t l_error_d32=0.0;
   double32_t l_error_limit_d32=0.0;
   double32_t l_output_voltage_d32=0.0;
   double32_t l_output_limit_voltage_d32=0.0;
   
   l_refvalue_d32 = *a_refvalue_d32;
   l_measure_d32 = *a_measuredvalue_d32;
   l_error_d32 = -l_measure_d32 + l_refvalue_d32;
   l_output_voltage_d32 = g_xi_d_d32 + KPI * l_error_d32;
   l_output_limit_voltage_d32 = l_output_voltage_d32;
   if(l_output_voltage_d32 > MAX_VOLTAGE)
   {
     l_output_limit_voltage_d32 =  MAX_VOLTAGE; 
   }
   else if(l_output_voltage_d32 < MIN_VOLTAGE)
   {
     l_output_limit_voltage_d32 = MIN_VOLTAGE;  
   }
   l_error_limit_d32 = l_output_voltage_d32 - l_output_limit_voltage_d32;
   g_xi_d_d32 = g_xi_d_d32 + (KI * l_error_d32) + (l_error_limit_d32);
   return l_error_d32;       
}


double32_t PIregulator_Speed(double32_t *a_refvalue_d32, double32_t *a_measuredvalue_d32)
{
   double32_t l_refvalue_d32=0.0;
   double32_t l_measure_d32=0.0;
   double32_t l_error_d32=0.0;
   double32_t l_error_limit_d32=0.0;
   double32_t l_output_voltage_d32=0.0;
   double32_t l_output_limit_voltage_d32=0.0;
   
   l_refvalue_d32 = *a_refvalue_d32;
   l_measure_d32 = *a_measuredvalue_d32;
   l_error_d32 = -l_measure_d32 + l_refvalue_d32;
   l_output_voltage_d32 = g_xi_speed_d32 + KPI * l_error_d32;
   l_output_limit_voltage_d32 = l_output_voltage_d32;
   if(l_output_voltage_d32 > MAX_VOLTAGE)
   {
     l_output_limit_voltage_d32 =  MAX_VOLTAGE; 
   }
   else if(l_output_voltage_d32 < MIN_VOLTAGE)
   {
     l_output_limit_voltage_d32 = MIN_VOLTAGE;  
   }
   l_error_limit_d32 = l_output_voltage_d32 - l_output_limit_voltage_d32;
   g_xi_speed_d32 = g_xi_speed_d32 + (KI * l_error_d32) + (l_error_limit_d32);
   return l_error_d32;       
}



