/*******************************************************************************
 svpwm.c

  Company:    Comstar Automotive Technologies.
	
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	
	

File Name:     svpwm.c

Summary:     This is the algorithm for space vector PWM

Description: 

 This program actually controls the voltage going on to the motor terminals.
	
 Created by 
 Date      : 14th May 2017
 Version   : 1.0 [BASE VERSION]
 
*******************************************************************************/

/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"
#include <math.h>

/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/
#define  PWMPERIOD      (10000UL)
#define  DCBUSVOLTAGE   (20u)
#define  SQRT_3			(1.732f)
#define  SQRT_3_BY_2    (0.866f)

/****************************************************************************
 * Section: File Scope Variables, Functions, Initializations
 * 
 *****************************************************************************/
void SvPwm(void);
extern void InverseClarkeTransform(voltage_T *a_voltage_ptr_s);
void SvPwm(void)
{
    //PortHigh();
 double32_t l_voltage_alpha_d32=g_voltage_s.g_voltage_measured_s.alpha_beta_axis.g_alpha_axis_d32;
 double32_t l_voltage_beta_d32=g_voltage_s.g_voltage_measured_s.alpha_beta_axis.g_beta_axis_d32;
 double32_t l_voltage_a_d32=0.0;
 double32_t l_voltage_b_d32=0.0;
 double32_t l_voltage_c_d32=0.0;
 struct sector_tag
 {
  uchar8_t l_a_u8;
  uchar8_t l_b_u8;
  uchar8_t l_c_u8;
  uchar8_t l_sector_u8;  
 }l_sector_s;
 
 struct time_tag
 {
   double32_t t[2];
 }l_time_s;
 struct xyz_tag
 {
     double32_t x;
     double32_t y;
     double32_t z;
 }l_xyz_s;
 double32_t l_temp_d32=0.0;
 struct dutycycle_tag
 {
     double32_t l_dutycycle_ta_on_d32;
     double32_t l_dutycycle_tb_on_d32;
     double32_t l_dutycycle_tc_on_d32;
 }l_dutycycle_s;
 
 
 l_voltage_a_d32 = g_voltage_s.g_voltage_measured_s.abcaxis.g_a_axis_d32;
 l_voltage_b_d32 = g_voltage_s.g_voltage_measured_s.abcaxis.g_b_axis_d32;
 l_voltage_c_d32 = g_voltage_s.g_voltage_measured_s.abcaxis.g_c_axis_d32;
  
 
 /* algorithm to find the sector s*/
 if(l_voltage_a_d32 > 0)
 {
   l_sector_s.l_a_u8 = 1;
 }
 else
 {
   l_sector_s.l_a_u8 = 0;
 }
 
 if(l_voltage_b_d32 > 0)
 {
   l_sector_s.l_b_u8 = 1;
 }
 else
 {
   l_sector_s.l_b_u8 = 0;
 }
 
 if(l_voltage_c_d32 > 0)
 {
   l_sector_s.l_c_u8 = 1;
 }
 else
 {
   l_sector_s.l_c_u8 = 0;
 }
 l_sector_s.l_sector_u8 = (l_sector_s.l_a_u8 + 2 * l_sector_s.l_b_u8 + 4 * l_sector_s.l_c_u8);
 l_temp_d32 = 5000;
 l_xyz_s.x = SQRT_3 * l_temp_d32 * l_voltage_beta_d32;
 l_xyz_s.y = (SQRT_3_BY_2)*l_temp_d32 * l_voltage_beta_d32 + 1.5 * l_temp_d32 * l_voltage_alpha_d32;
 l_xyz_s.z = (SQRT_3_BY_2)* l_temp_d32 * l_voltage_beta_d32 - (1.5)*l_temp_d32 * l_voltage_alpha_d32;
 
 if(0x01 == l_sector_s.l_sector_u8)  
 {
  l_time_s.t[0] =  l_xyz_s.z; 
  l_time_s.t[1] =  l_xyz_s.y; 
 }
 else if(0x02 == l_sector_s.l_sector_u8)
 {
  l_time_s.t[0] =  l_xyz_s.y; 
  l_time_s.t[1] = -l_xyz_s.x;   
 }
 else if(0x03 == l_sector_s.l_sector_u8)
 {
  l_time_s.t[0] = -l_xyz_s.z; 
  l_time_s.t[1] =  l_xyz_s.x;   
 }
 else if(0x04 == l_sector_s.l_sector_u8)  
 {
  l_time_s.t[0] = -l_xyz_s.x; 
  l_time_s.t[1] =  l_xyz_s.z; 
 }
 else if(0x05 == l_sector_s.l_sector_u8)
 {
  l_time_s.t[0] =  l_xyz_s.x; 
  l_time_s.t[1] = -l_xyz_s.y;   
 }
 else if(0x06 == l_sector_s.l_sector_u8)
 {
  l_time_s.t[0] = -l_xyz_s.y; 
  l_time_s.t[1] = -l_xyz_s.z;   
 }
 /*
  saturation timings
  */
 if((l_time_s.t[0] + l_time_s.t[1]) > PWMPERIOD)
 {
     l_time_s.t[0] = (l_time_s.t[0] * PWMPERIOD)/(l_time_s.t[0] + l_time_s.t[1]);
     l_time_s.t[1] = (l_time_s.t[1] * PWMPERIOD)/(l_time_s.t[0] + l_time_s.t[1]);
 }
 
 l_dutycycle_s.l_dutycycle_ta_on_d32 = (PWMPERIOD-l_time_s.t[0]-l_time_s.t[1])*0.5;
 l_dutycycle_s.l_dutycycle_tb_on_d32 = l_dutycycle_s.l_dutycycle_ta_on_d32 + l_time_s.t[0];
 l_dutycycle_s.l_dutycycle_tc_on_d32 = l_dutycycle_s.l_dutycycle_tb_on_d32 + l_time_s.t[1];

 switch (l_sector_s.l_sector_u8)
 {
     case 0x01:
         PDC1 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
      break;
     case 0x02:
         PDC1 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
      break;
     case 0x03:
         PDC1 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
      break;
     case 0x04:
         PDC1 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
      break;
     case 0x05:
         PDC1 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
      break;
     case 0x06:
         PDC1 =  l_dutycycle_s.l_dutycycle_tb_on_d32;
         PDC2 =  l_dutycycle_s.l_dutycycle_tc_on_d32;
         PDC3 =  l_dutycycle_s.l_dutycycle_ta_on_d32;
      break;
     
     default:
         break;
 }
 //PortLow();
}

