/*******************************************************************************
 transforms.c

  Company:    Comstar Automotive Technologies.
	
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	
	

File Name:   transforms.c

Summary:     This contains the transformations 

Description: 

 this is the heart of all transformations. It consists of park, clarke, Inverse park, Inverse
 * clarke transformations.
	
 Created by 
 Date      : 14th May 2017
 Version   : 1.0 [BASE VERSION]
 
*******************************************************************************/

/****************************************************************************** 
 ******************************************************************************
 Section: Included Files
 ******************************************************************************
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"
#include <math.h>

/*****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************/
#define  SQRT_3			1.732f

#define CLARKECONSTANT  {                              \
                         {1.0,-0.5,-0.5},              \
                         {0,0.8666,-0.8666}, \
                         {0.5,0.5,0.5}                 \
                        }                              \

double g_clarke_matrix_a[3][3] = CLARKECONSTANT;
extern const int sinelut[1000];
extern struct g_angleinfo_tag_s
{
    sint16_t g_sinevalue_s16;
    sint16_t g_cosvalue_s16;
}g_angleinfo_s;

void ClarkeTransform(current_T *a_ptr_s);
void ParkTransform(current_T *a_current_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s);
void InverseParkTransform(voltage_T *a_voltage_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s);
void InverseClarkeTransform(voltage_T *a_voltage_ptr_s);

/* it is verified  ialpha = ia
   ibeta = (1/sqrt(3)) * (ib - ic) 
 */
void ClarkeTransform(current_T *a_ptr_s)
{
    double32_t l_measured_a_axis_d32=0.0;
    double32_t l_measured_b_axis_d32=0.0;
    double32_t l_measured_c_axis_d32=0.0;
    
    l_measured_a_axis_d32 = a_ptr_s ->g_current_measured_s.abcaxis.g_a_axis_d32;
    l_measured_b_axis_d32 = a_ptr_s ->g_current_measured_s.abcaxis.g_b_axis_d32;
    l_measured_c_axis_d32 = a_ptr_s ->g_current_measured_s.abcaxis.g_c_axis_d32;   
    /* alpha is same as a axis */
    a_ptr_s->g_current_measured_s.alpha_beta_axis.g_alpha_axis_d32 = l_measured_a_axis_d32;
    a_ptr_s->g_current_measured_s.alpha_beta_axis.g_beta_axis_d32 = 0.5773f*(l_measured_b_axis_d32 - l_measured_c_axis_d32);
}
/*
 * isd = ialpha * cos(theta) + ibeta * sin(theta)
 * isq = -ialpha * sine(theta) + ibeta * cos(theta);
 */

void ParkTransform(current_T *a_current_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s)
{
    double32_t l_current_aplha_d32=0.0;
    double32_t l_current_beta_d32=0.0;
    //sint16_t l_motor_theta_s16=0;
    
    l_current_aplha_d32 = a_current_ptr_s->g_current_measured_s.alpha_beta_axis.g_alpha_axis_d32;
    l_current_beta_d32 = a_current_ptr_s->g_current_measured_s.alpha_beta_axis.g_beta_axis_d32;
    //l_motor_theta_s16 = a_measured_par_ptr_s->g_measured_position_theta_s16;
    
    a_current_ptr_s->g_current_measured_s.d_q_axis.g_d_axis_d32 = l_current_aplha_d32 *0.001f*g_angleinfo_s.g_cosvalue_s16\
                   + l_current_beta_d32*0.001f*g_angleinfo_s.g_sinevalue_s16;
    a_current_ptr_s->g_current_measured_s.d_q_axis.g_q_axis_d32 = l_current_beta_d32*0.001f*g_angleinfo_s.g_cosvalue_s16 \
                   - l_current_aplha_d32 * 0.001f*g_angleinfo_s.g_sinevalue_s16;    
}

void InverseParkTransform(voltage_T *a_voltage_ptr_s, motor_measured_parametres_T *a_measured_par_ptr_s)
{
  double32_t l_voltage_d_axis_d32=a_voltage_ptr_s->g_voltage_measured_s.d_q_axis.g_d_axis_d32;
  double32_t l_voltage_q_axis_d32=a_voltage_ptr_s->g_voltage_measured_s.d_q_axis.g_q_axis_d32;
  //sint16_t l_theta_s16=a_measured_par_ptr_s->g_measured_position_theta_s16;
  
  a_voltage_ptr_s->g_voltage_measured_s.alpha_beta_axis.g_alpha_axis_d32 = l_voltage_d_axis_d32 * 0.001f * g_angleinfo_s.g_cosvalue_s16 - l_voltage_q_axis_d32 *0.001f*g_angleinfo_s.g_sinevalue_s16 ; 
  a_voltage_ptr_s->g_voltage_measured_s.alpha_beta_axis.g_beta_axis_d32 = l_voltage_q_axis_d32 * 0.001f * g_angleinfo_s.g_cosvalue_s16 \
                 + l_voltage_d_axis_d32 * 0.001f * g_angleinfo_s.g_sinevalue_s16;
}

/*
 * Verified the logic is
 * va = vsbeta
 * vb = 0.5*(sqrt(3)*valpha - vbeta);
 * vc = 0.5 * (-sqrt(3)*valpha - vbeta)
 */
void InverseClarkeTransform(voltage_T *a_voltage_ptr_s)
{
    double32_t l_voltage_alpha_d32=0.0;
    double32_t l_voltage_beta_d32=0.0;
    
    l_voltage_alpha_d32 = a_voltage_ptr_s->g_voltage_measured_s.alpha_beta_axis.g_alpha_axis_d32;
    l_voltage_beta_d32 = a_voltage_ptr_s->g_voltage_measured_s.alpha_beta_axis.g_beta_axis_d32;
    g_voltage_s.g_voltage_measured_s.abcaxis.g_a_axis_d32 = (l_voltage_beta_d32);
    g_voltage_s.g_voltage_measured_s.abcaxis.g_b_axis_d32 = (SQRT_3 * l_voltage_alpha_d32 - l_voltage_beta_d32 )*0.5;
    g_voltage_s.g_voltage_measured_s.abcaxis.g_c_axis_d32 = (-l_voltage_beta_d32 - SQRT_3 * l_voltage_alpha_d32)*0.5; 
}


