

/******************************************************************************
 *****************************************************************************
 Section: Included Files
 *****************************************************************************
 *****************************************************************************/

#include<xc.h>
#include "types.h"
#include "motorvariables.h"

 
/*****************************************************************************
 *****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************
 *****************************************************************************/
#define ON (1u)
#define OFF (0u)

/* Main Function */
/*********************************************************************
 * Function: ADC_Func_dl()
 *
 * Overview: The10-bit high-speed analog-to-digital converter (A/D) allows conversion of an analog input signal 
  to a 10-bit digital number. This moduleis based on a Successive Approximation Register (SAR) architecture,
  and provides a maximum sampling rate of 500 ksps. The A/D module has 16 analog inputs which are multiplexed 
  into four sample and hold amplifiers. The output of the sample and hold is the input intothe converter, 
  which generates the result. The analog reference voltages are software selectable to either the device
  supply voltage(AVDD/AVSS) or the voltage level on the (VREF+/VREF-) pin
 * 
 * PreCondition: None
 *
 * Input: ADC interrupt generation after completing of every nth sample/conversion operation 
 ? No of analog Channels to be enabled .
 *
 * Output:   Read the ADC conversion result of the nth inputs.
 *
 ********************************************************************/
/*LDRA_INSPECTED 65 D: Void function has no side effects: Here this function have only have registers */
void initAdc1(void)
{/* Set port configuration */
    
 //ANSELA = ANSELB = ANSELC = ANSELD = ANSELE = ANSELG = 0x0000;
 //ANSELBbits.ANSB5 = 1;// Ensure AN5/RB5 is analog
    
 _TRISA7=ON;      /* Phase1 current */
_TRISA11=ON;     /*  PhaseY_volt   */
_TRISA12=ON;     /*Phase R_volt */

/* Enable B port configuration */
_TRISB0=OFF;      /*Machine Temperature */
_TRISB1=OFF;     /*Board Temperature*/


/* Enable C port configuration */
_TRISC11=ON;  /*12V Bat Monitoring*/
_TRISC4=ON;   /*Phase 2 Current */
_TRISC5=ON;  /*Field volt monitoring*/
//_TRISC9=ON; /*12V buck monitoring */
_TRISC13 = OFF;


/* Enable E port configuration */
_TRISE12=ON;  /*Phase B volt */
_TRISE13=ON;  /*Field volt monitoring*/
_TRISE14=ON;  /*5V monitoring */
_TRISE15=ON;  /*Field Current monitoring*/

/* Enable G port configuration*/
_TRISG6=ON;  /*Phase3 current monitoring */
_TRISG7=ON;  /*Bus current monitoring */


ANSELAbits.ANSA7=ON;  /* Phase1 current */
ANSELAbits.ANSA11=ON; /*  PhaseY_volt   */
ANSELAbits.ANSA12=ON; /*Phase R_volt */

ANSELBbits.ANSB0=OFF;  /*Machine Temperature */
ANSELBbits.ANSB1=OFF;  /*Board Temperature*/

ANSELCbits.ANSC11=ON;  /*12V Bat Monitoring*/
ANSELCbits.ANSC4=ON;    /*Phase 2 Current */
ANSELCbits.ANSC5=ON;   /*Field volt monitoring*/

//ANSELCbits.ANSC9=ON;  /*12V buck monitoring */
ANSELCbits.ANSC9=OFF;  // For Diagnostic LED

ANSELEbits.ANSE12=ON;  /*Phase B volt */
ANSELEbits.ANSE13=ON;  /*Field volt monitoring*/
ANSELEbits.ANSE14=ON;   /*5V monitoring */
ANSELEbits.ANSE15=ON;  /*Field Current monitoring*/

ANSELGbits.ANSG6=ON;  /*Phase3 current monitoring */
ANSELGbits.ANSG7=ON;  /*Bus current monitoring */

 ANSELCbits.ANSC6=ON;   /*48v Monitoring*/
_TRISC6=ON;  /*48v Monitoring*/

 _TRISC10=ON; /*48v Monitoring */
 ANSELCbits.ANSC10=ON;  /*48v Monitoring */
 
 _TRISA7=ON;      /* Phase1 current */ 
 ANSELAbits.ANSA7=ON;  /* Phase1 current */
  

 /* Initialize and enable ADC module */
 
 
 AD1CON1 = 0x0000;
 AD1CON1bits.AD12B=1;
 AD1CON2 = 0x0000;
 AD1CON3 = 0x000F;
 AD1CON4 = 0x0000;
 AD1CHS0 = 0x0002;
 AD1CHS123 = 0x0000;
 AD1CSSH = 0x0000;
 AD1CSSL = 0x0000;
 AD1CON1bits.ADON = 1;
 Delay_us(20);
}


void Delay_us(unsigned int delay)
{
    uchar8_t i;
    for (i = 0; i < delay; i++)
    {__asm__ volatile ("repeat #39");
     __asm__ volatile ("nop");
    }
}


