/*******************************************************************************
 adc.c

  Company:    Comstar Automotive Technologies.
	
  Copyright (c) Comstar Automotive Technologies.  All rights reserved.

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL COMSTAR OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.	
	

File Name:     adc.c

Summary:     this is the adc driver for ADC module

Description: This driver file will trigger adc module on pwm interrrupt

 This program is the control of the foc this is called from pwm interrupt every 60us. 
	
 Created by 
 Date      : 14th May 2017
 Version   : 1.0 [BASE VERSION]
 
*******************************************************************************/

/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"

/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/

/****************************************************************************
 * Section: File Scope Variables, Functions, Initializations
 * 
 *****************************************************************************/

/****************************************************************************
  Section: Function Declarations, extern declarations
 *****************************************************************************/
void AdcInitialization(void);
void ConvertAdcCountToVoltage(uint16_t a_current_count_a_u16,uint16_t a_current_count_b_u16);


void AdcInitialization(void)
{
   ANSELBbits.ANSB1 = 1;                                                     /* Ensure AN3/RB1 is analog*/
   ANSELBbits.ANSB2 = 1;                                                     /* Ensure AN4/RB2 is analog*/
   AD1CSSL = 0x18;                      /* Scan mode */
   AD1CSSH = 0x00;
   AD1CHS0 = 0x00;
   AD1CHS123 = 0x00;
   AD1CON3 = 0x0405;
   AD1CON2 = 0x0404;
   //AD1CON1 = 0x8014;    /* pwm1 interrupt*/
   AD1CON1 = 0x8044;
  _AD1IP = 2;                          /*Interrupt Priority : 2*/
  _AD1IF = 0;
  _AD1IE = 1;                          /* Enable ADC Interrupt */
}

void adc_init( void)
{	
AD1CON4 = 0x0000;                                                           /* DMA is not used */
AD1CSSH = 0x0000;                                                           /* RESET OF ADCx INPUT SCAN SELECT REGISTER HIGH */
AD1CSSL = 0x0000;                                                           /* Reset of ADCx INPUT SCAN SELECT REGISTER LOW   */
AD1CHS0 = 0x0000;                                                           /* if CSCNA =1 , AD1CHS0 no effect  */

/* Enable A port configuration */


_TRISB1 = 1;
_TRISB2 = 1;
ANSELBbits.ANSB1 = 1;                                                     /* Ensure AN3/RB1 is analog*/
ANSELBbits.ANSB2 = 1;                                                     /* Ensure AN4/RB2 is analog*/

/*-----------------------AD1CON1 - Control registers----------------------------------------------*/				   
 AD1CON1bits.ASAM  = 1;                  /* Sampling begins immediately after last conversion; SAMP bit is auto-set */ 
 AD1CON1bits.SSRCG = 0;                 /* Sample Trigger Source Group bit */
 AD1CON1bits.SSRC  = 7 ;     /* Internal counter ends sampling and starts conversion (auto-convert)*/
/*-----------------------AD1CON2 - Control registers----------------------------------------------*/	
AD1CON2bits.ALTS   = 0;                /* Always uses channel input selects for Sample MUXA */
AD1CON2bits.BUFM   = 0;                /* Always starts filling the buffer from the Start address */
AD1CON2bits.SMPI   = 9;/* Sample and Conversion Operation bits..ADC interrupt is generated at the completion of every 8th sample/conversion OPeration*/
AD1CON2bits.BUFS   = 0;   /* ADC is currently filling the first half of the buffer; the user application should access data in the second half of the buffer*/
AD1CON2bits.CHPS   = 0;                 /* Converts CH0 */
AD1CON2bits.CSCNA  = 1;                  /* Scans inputs for CH0+ during Sample A bit */
/*-----------------------AD1CON3 - Control registers----------------------------------------------*/
AD1CON3bits.ADCS = 0 ;                                            /* Auto-Sample Time bit & ADCx Conversion Clock Select bits*/
AD1CON3bits.SAMC = 0 ;
/*-------------------------------------------------------------------------------------------------*/
AD1CSSL = 0x18; /* Selects ANx for input scan */
AD1CON1bits.ADON = 1;                     /* ADC module is operating    */
IFS0bits.AD1IF = 0;                      /* Clear the ADC Interrupt flag */
IEC0bits.AD1IE = 1;                       /* Enable the ADC Interrupt     */
}


#if 0
void __attribute__((__interrupt__,__auto_psv__)) _AD1Interrupt(void)
{
  uint16_t l_current_count_a_u16=0;
  uint16_t l_current_count_b_u16=0;
  
  l_current_count_a_u16 = ADC1BUF0;      /* Copy value from ADC buffer  */
  l_current_count_b_u16 = ADC1BUF1;      /* Copy value from ADC buffer  */
  ConvertAdcCountToVoltage(l_current_count_a_u16,l_current_count_b_u16);
  
  AD1CON1 = 0x44;                      /* Must cancel current sampling */
  AD1CON1 = 0x8044;                    /* Re-scoet ADC configuration for triggered sampling */
  TogglePort();
  _AD1IF = 0;                        /* Re-enable interrupt */
}
#endif

static void __attribute__((__interrupt__)) _AD1Interrupt(void) 
{	
unsigned int g_adc_buff[9];    /* Creating array for the analog channels */

g_adc_buff[0] = ADC1BUF0;
                                             /* Reading the HallA sensor ADC  value and storing it in a buffer  */	
g_adc_buff[1] = ADC1BUF1;
                                            /* Reading the HallB sensor ADC  value and storing it in a buffer  */


g_adc_buff[2] = ADC1BUF2;

g_adc_buff[3] = ADC1BUF3;
        
g_adc_buff[4] = ADC1BUF4;                                                   /* Reading the fis1 sensor ADC  value and storing it in a buffer   */
	
g_adc_buff[5] = ADC1BUF5;                                                   /* Reading the fis2 sensor ADC  value and storing it in a buffer   */
	
g_adc_buff[6] = ADC1BUF6;                                                   /* Reading the Current  sensor ADC  value and storing it in a buffer     */
	
g_adc_buff[7] = ADC1BUF7;                                                   /* Reading the Board Temperature  ADC  value and storing it in a buffer  */
	
//g_adc_batteryvoltage_u16 = ADC1BUF8;                                                   /* Reading the Battery voltage  ADC  value and storing it in a buffer    */
    
IFS0bits.AD1IF = 0;                                                     /* Reset of ADC Interrupt*/
}


void ConvertAdcCountToVoltage(uint16_t a_current_count_a_u16,uint16_t a_current_count_b_u16)
{
    /* for every 0.1v difference the current is 1 amp */
    double32_t l_ref_voltage_d32 = 2.5;
    double32_t l_diff_voltage_d32=0.0;
    double32_t l_current_a_d32=0.0;
    double32_t l_current_b_d32=0.0;
     
    l_current_a_d32 = (5.0 * a_current_count_a_u16)/(1024.0);
    l_current_b_d32 = (5.0 * a_current_count_b_u16)/(1024.0);
    
    l_diff_voltage_d32 = (l_current_a_d32 - l_ref_voltage_d32);
    g_current_s.g_current_measured_s.abcaxis.g_a_axis_d32 =  l_diff_voltage_d32 * 10.0;       
    l_diff_voltage_d32 = (l_current_b_d32 - l_ref_voltage_d32);
    g_current_s.g_current_measured_s.abcaxis.g_b_axis_d32 =  l_diff_voltage_d32 * 10.0;           
}