/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include "types.h"
#include "xc.h"
#include "can.h"

/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/
#define  NUM_OF_ECAN_BUFFERS 	32

/****************************************************************************
 * Section: File Scope Variables, Functions, Initializations
 * 
 *****************************************************************************/
typedef struct
{		
uint16_t l_receive_id_u16;
uchar8_t l_receive_dlc_u8;
uchar8_t l_can_input_u8A[8];
}CANRXID;
CANRXID g_canRxMessage;
uchar8_t g_datarxed_u8; /*test purpose*/
unsigned int ecan1MsgBuf[NUM_OF_ECAN_BUFFERS][8]
__attribute__((aligned(NUM_OF_ECAN_BUFFERS * 16)));

extern float32_t g_adc_phase1_ampl_volt_d32;
extern float32_t g_adc_phase2_ampl_volt_d32;
extern float32_t g_adc_phase3_ampl_volt_d32;
extern uchar8_t g_phase_Current_Sign_u8;

extern uint16_t g_adc_phase1_count_u16;
extern uint16_t g_adc_phase2_count_u16;
extern uint16_t g_adc_phase3_count_u16;

/****************************************************************************
  Section: Function Declarations, extern declarations
 *****************************************************************************/
void Can_Transmit(uint16_t a_id_u16, uchar8_t *a_data_u8, uchar8_t a_dlc_u8);
void Can_Init(void);


void Can_Init(void) 
{         

_TRISF1     = OUTPUT_PORT;
_TRISF0     = INPUT_PORT;
//
// remap the CAN module to the proper pins on the board
//
RPINR26                 =   PORT_RPI96;         // connect CAN RX to RPI96
RPOR9                   =   PORT_RP97;         // connect CAN TX to RP97 
C1CTRL1bits.REQOP = SET_CONF_MODE;
while(C1CTRL1bits.OPMODE != SET_CONF_MODE);
C1CTRL1bits.WIN         =   BUF_WIN;
C1CFG1bits.SJW 			= 	SJW_1TQ;    		/* synchronized jump width = 00 */
C1CFG1bits.BRP 			= 	BRP_VAL;            /* baud rate 5/fcan		*/
C1CFG2bits.WAKFIL 		= 	WAKFIL_NOLINFIL;   	/* can bus line filter is not used for wake up */
C1CFG2bits.SEG2PH 		= 	SEG2PH_3TQ;   		/* phase buffer segment 5 = 101 */
C1CFG2bits.SEG2PHTS 	= 	SEG2PHTS_FREEPGM;   /* phase segment 2 time select = freely programmable */
C1CFG2bits.SAM 			= 	SAM_THRTIM;    		/* sample can bus line = 0 (1x) */
C1CFG2bits.SEG1PH 		= 	SEG1PH_3TQ;   		/* phase buffer segment 1 = 101 */
C1CFG2bits.PRSEG 		= 	PRSEG_3TQ;    		/* propagation time = 101 */
C1FCTRLbits.DMABS   = DMA_32BUF;                  /* No FIFO, 32 Buffers */
C1FCTRLbits.FSA     = FSA_32RXBUF;       
DMA0CONbits.SIZE    = SIZE_WORD;
DMA0CONbits.DIR     = DIR_RAMTOPER;
DMA0CONbits.AMODE   = AMODE_PER;
DMA0CONbits.MODE    = MODE_CON;
DMA0REQ             = CAN_TXDATA;
DMA0CNT             = TRANS_CNT;
DMA0PAD             = (volatile unsigned int)&C1TXD;
DMA0STAL            = (unsigned int) &ecan1MsgBuf;
DMA0STAH            = (unsigned int) &ecan1MsgBuf;
DMA0CONbits.CHEN    = DMA_CHEN;
DMA1CONbits.SIZE    = SIZE_WORD;
DMA1CONbits.DIR     = DIR_PERTORAM;
DMA1CONbits.AMODE   = AMODE_PER;
DMA1CONbits.MODE    = MODE_CON;
DMA1REQ             = CAN_RXDATA;
DMA1CNT             = TRANS_CNT;
DMA1PAD             = (volatile unsigned int)&C1RXD;
DMA1STAL            = (unsigned int) &ecan1MsgBuf;
DMA1STAH            = (unsigned int) &ecan1MsgBuf;      
DMA1CONbits.CHEN    = DMA_CHEN;
/* Configure Message Buffer 0 for Transmission and assign priority */
C1TR01CONbits.TXEN0  = TX_BUF;
C1TR01CONbits.TX0PRI = HIGH_PRIO;
/* At this point the ECAN1 module is ready to transmit a message. Place the ECAN module in
Normal mode. */
C1CTRL1bits.WIN         =   FILT_WIN;
/* Select Acceptance Filter Mask 0 for Acceptance Filter 0 */
C1FMSKSEL1bits.F0MSK    =   FIL_MASK0 ;
/* Configure Acceptance Filter Mask 0 register to mask SID<2:0>
* Mask Bits (11-bits) : 0b111 1111 1000 */
C1RXM0SIDbits.SID       =   MASK_SID ;
/* Configure Acceptance Filter 0 to match standard identifier
Filter Bits (11-bits): 0b011 1010 xxx with the mask setting, message with SID
range 0x1D0-0x1D7 will be accepted by the ECAN module. */
C1RXF0SIDbits.SID       =   FIL_SID;
/* Acceptance Filter 0 to check for Standard Identifier */
C1RXM0SIDbits.MIDE      =   MASK_EXIDE;
C1RXF0SIDbits.EXIDE     =   FIL_EXIDE;
/* Acceptance Filter 0 to use Message Buffer 10 to store message */
C1BUFPNT1bits.F0BP      =   RX_BUFFER;
/* Filter 0 enabled for Identifier match with incoming message */
C1FEN1bits.FLTEN0       =   EN_FILTER;
/* Clear Window Bit to Access ECAN
* Control Registers */
C1CTRL1bits.WIN         =   BUF_WIN;
/* Place the ECAN module in normal
* mode. */
C1CTRL1bits.REQOP = SET_NORMAL_MODE;
while(C1CTRL1bits.OPMODE != SET_NORMAL_MODE);
/* The following code shows one example of how the application can wait
for a message to be received in message buffer 10 */

C1INTF 					= 	CAN_INF_DISABLE;			/* Reset all The CAN Interrupts */
IFS2bits.C1IF           =   DISABLE;                      /* clear interrupt flag */
C1INTE 					= 	CAN_INT_ENREG;     /* Enable all CAN interrupt sources */
IEC2bits.C1IE 			= 	ENABLE;  
}
void Can_Transmit(uint16_t a_id_u16, uchar8_t *a_data_u8A, uchar8_t a_dlc_u8)
{    
uint16_t l_id_u16 = a_id_u16;
uchar8_t l_dlc_u8 = a_dlc_u8;
uchar8_t *l_data_u8A = a_data_u8A;
uint16_t l_can_output[8];
uchar8_t l_index;

	/* Write to message buffer 0 */
/* CiTRBnSID = 0bxxx1 0010 0011 1100
IDE = 0b0
SRR = 0b0
SID<10:0>= 0b100 1000 1111 */
ecan1MsgBuf[0][0] = (l_id_u16<<2 ) & 0x1FFC;
/* CiTRBnEID = 0bxxxx 0000 0000 0000
EID<17:6> = 0b0000 0000 0000 */
ecan1MsgBuf[0][1] = 0x0000;
/* CiTRBnDLC = 0b0000 0000 xxx0 1111
EID<17:6> = 0b000000
RTR = 0b0
RB1 = 0b0
RB0 = 0b0
DLC = 0b1111 */
ecan1MsgBuf[0][2] = l_dlc_u8;
/* Write message data bytes */
for (l_index = 0; l_index < l_dlc_u8; l_index++)
{
  l_can_output[l_index] = *l_data_u8A++;
}
ecan1MsgBuf[0][3] = (l_can_output[1] <<8)| l_can_output[0];
ecan1MsgBuf[0][4] = (l_can_output[3] <<8)| l_can_output[2];
ecan1MsgBuf[0][5] = (l_can_output[5] <<8)| l_can_output[4];
ecan1MsgBuf[0][6] = (l_can_output[7] <<8)| l_can_output[6];
/* Request message buffer 0 transmission */
C1TR01CONbits.TXREQ0 = TXREQ_ENABLE;
}
  
void Can_Receive(CANRXID *a_receivemsg_S)
{
a_receivemsg_S -> l_receive_id_u16      = (ecan1MsgBuf[10][0] & 0x1FFC) >> 2;
a_receivemsg_S -> l_receive_dlc_u8      = ecan1MsgBuf[10][2] & 0x000F;
a_receivemsg_S -> l_can_input_u8A[0] =  ( ecan1MsgBuf[10][3] & 0x00FF );
a_receivemsg_S -> l_can_input_u8A[1] = ( ecan1MsgBuf[10][3] & 0xFF00 ) >> 8;
a_receivemsg_S -> l_can_input_u8A[2] =  ( ecan1MsgBuf[10][4] & 0x00FF );
a_receivemsg_S -> l_can_input_u8A[3] =   ( ecan1MsgBuf[10][4] & 0xFF00 ) >> 8;
a_receivemsg_S -> l_can_input_u8A[4] =  ( ecan1MsgBuf[10][5] & 0x00FF );
a_receivemsg_S -> l_can_input_u8A[5] =  ( ecan1MsgBuf[10][5] & 0xFF00 ) >> 8;
a_receivemsg_S -> l_can_input_u8A[6] = ( ecan1MsgBuf[10][6] & 0x00FF );
a_receivemsg_S -> l_can_input_u8A[7] =  ( ecan1MsgBuf[10][6] & 0xFF00 ) >> 8;
g_datarxed_u8 = TRUE;        
}
 void __attribute__((interrupt, no_auto_psv)) _DMA1Interrupt(void)
{
    IFS0bits.DMA1IF = DISABLE; // Clear the DMA0 Interrupt Flag;
}
void __attribute__((interrupt, no_auto_psv)) _DMA0Interrupt(void)
{
    IFS0bits.DMA0IF = DISABLE; // Clear the DMA0 Interrupt Flag;
}
void __attribute__((interrupt, no_auto_psv))_C1Interrupt(void)
{
IFS2bits.C1IF = DISABLE; // clear interrupt flag
if (C1INTFbits.TBIF)
{
    C1INTFbits.TBIF = DISABLE;
}
if (C1INTFbits.RBIF)
{
  Can_Receive(&g_canRxMessage); 
  if (C1RXFUL1bits.RXFUL10 == 1)
  {
        C1RXFUL1bits.RXFUL10 = 0;
  }
  if (C1RXOVF1bits.RXOVF10 == 1)
  {
        C1RXOVF1bits.RXOVF10 = 0;
  }
  C1INTFbits.RBIF = DISABLE;
} 
}     

extern int ControlOutput;
extern uint16_t Phase;
extern uint16_t g_period_u16;
extern uchar8_t g_flag_u8;
extern uchar8_t g_sector;
extern signed int PhaseInc;
void can_transmit()
{
    uint16_t l_id_u16=0;
    uint16_t l_id1_u16=0;
    uchar8_t l_dlc_u8=0;
    uchar8_t l_dlc1_u8=0;
    uchar8_t l_data_array[8]={0};
    uchar8_t l_data1_array[8]={0};
    uchar8_t *l_data_P;
    uchar8_t *l_data1_P;
    uint16_t l_delay_u16;    
    l_id_u16=0x500;
    l_dlc_u8=8;
    l_data_P=&l_data_array[0];
    
    l_id1_u16 =0x501;
    l_dlc1_u8=8;
    l_data1_P=&l_data1_array[0];
    l_data_array[0]= g_period_u16 >> 8;
    l_data_array[1]=  g_period_u16;
    l_data_array[2]=  g_flag_u8;
    l_data_array[3]=  g_sector;
    l_data_array[4]=  PhaseInc >> 8;
    l_data_array[5]=  PhaseInc;
    l_data_array[6]=  0xff;//ControlOutput >> 8;
    l_data_array[7]= 0xff; //ControlOutput;
    Can_Transmit(l_id_u16, l_data_P,l_dlc_u8); 
//for(l_delay_u16=0;l_delay_u16<500;l_delay_u16++);
//Can_Transmit(l_id1_u16, l_data1_P,l_dlc1_u8 ); 
//for(l_delay_u16=0;l_delay_u16<500;l_delay_u16++);
    
}