
/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"
/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/


uint16_t Phase;
signed int PhaseInc; // Delta increments of the Phase variable, calculated 
                     // in the TIMER1 interrupt and used in 
                     // the PWM interrupt (each 50 us)
int ControlOutput = 0;	// Controller output, used as a voltage output, 
                        // use its sign for the required direction

void PwmInitialization(void);
extern void FocAlgorithm(void);
/*
 center aligned 
 interrupt based */
//Half of the PWM Deadtime; The effective deadtime written in ALTDTR registers is 2*ALTDTR_DIV2
#define	ALTDTR_DIV2	120
#define FCY  70000000	 // xtal = 8Mhz; with PLL -> 70 MIPS
#define FPWM 20000		 // 20 kHz, so that no audible noise is present.
void PwmInitialization(void)
{
    PHASE1 = (FCY/FPWM - 1);
	PHASE2 = (FCY/FPWM - 1);
	PHASE3 = (FCY/FPWM - 1);

	//IOCON1 = 0;
	//IOCON2 = 0;
	//IOCON3 = 0;
    IOCON1bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON1bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON1bits.PMOD = 0;   					/* Select complementary mode Output PWM mode */
    
     IOCON2bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON2bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON2bits.PMOD = 0;   					/* Select Complementary Output PWM mode */
    
    IOCON3bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON3bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON3bits.PMOD = 0;   					/* Select Complementary Output PWM mode */
    

	DTR1 = 0x0000;	// 2 us of dead time
	DTR2 = 0x0000;	// 2 us of dead time
	DTR3 = 0x0000;	// 2 us of dead time

	ALTDTR1 = 2*ALTDTR_DIV2;	// 2 us of dead time		
	ALTDTR2 = 2*ALTDTR_DIV2;	// 2 us of dead time
	ALTDTR3 = 2*ALTDTR_DIV2;	// 2 us of dead time
	//Note: ALTDTR_DIV2 define in svm.c should be half of ALTDTRx


	PTCON2 = 0x0000;	// Divide by 1 to generate PWM

	PWMCON1 = 0x0604;	// Enable PWM output pins and configure them as 
	PWMCON2 = 0x0204;	// complementary mode
	PWMCON3 = 0x0204;

	PDC1 = PHASE1 / 2;	// Initialize as 0 voltage
	PDC2 = PHASE2 / 2;	// Initialize as 0 voltage
	PDC3 = PHASE3 / 2;	// Initialize as 0 voltage

	SEVTCMP = 1;	    // Enable triggering for ADC
	
	//fault enabled	
	FCLCON1 = 0x0005;
	FCLCON2 = 0x0005;
	FCLCON3 = 0x0005;
	
    IFS5bits.PWM1IF = 0;    /* Clear PWM1 Interrupt Flag */
    IEC5bits.PWM1IE = 1;    /* PWM1 Interrupt Enable */
	PTCON = 0x8000;		// start PWM

	return;				 

}

void PwmInitUpdated(void)
{
    
   IFS5bits.PWM1IF = 0;    /* Clear PWM1 Interrupt Flag */
   IEC5bits.PWM1IE = 1;    /* PWM1 Interrupt Enable */

  /* ~~~~~~~~~~~~~~~~~~~~~~ PWM1 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    IOCON1bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON1bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON1bits.PMOD = 0;   					/* Select complementary mode Output PWM mode */
    
    PWMCON1bits.CAM = 1;                    /* Select Center-aligned mode */
    PWMCON1bits.ITB = 1;                    /* Select Independent timebase mode (required for
                                                center-aligned mode) */
    PWMCON1bits.TRGIEN = 1;
    
    TRGCON1 = 0x0000;       /* PWM Trigger interrupt generated every
                              // PWM cycle
                              // Trigger generated 32nsec from beginning
                              // of PWM period */
    TRIG1 = 0x0008;         /* TRIGx value of 0 does not produce
                              // a trigger. So choose smallest possible
                              // Trigger time = TRIG1*1.05nsec = 8.4 nsec */
	
    PHASE1 = PHASE_A_FRE;             				/* In Center-aligned mode the effective period of 
                                               the PWM signal is twice of the value in the 
                                               PHASEx register. So to obtain signal of 400kHz
                                               (or 2.5us period), the PHASE1 register should be
                                               configured for a period of 1.25us.
                                               PHASE1 = ((1.25us) / 1.04ns) = 1202. So effective
                                               period is 2.5us. */
    PDC1 = PHASE_A_FRE * (DUTY_PHASE_A/100.0);           /* In Center-aligned mode the effective duty cycle of 
                                               the PWM signal is twice of the value in the 
                                               PDCx register. To achieve 50% duty cycle configure
                                               PDCx register to 0.625us.
                                               PDC1 = ((0.625us) / 1.04ns) = 601. */
    
    /* ~~~~~~~~~~~~~~~~~~~~~~ PWM2 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    IOCON2bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON2bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON2bits.PMOD = 0;   					/* Select Complementary Output PWM mode */
    PWMCON2bits.CAM = 1;                    /* Select Center-aligned mode */
    PWMCON2bits.ITB = 1;                    /* Select Independent timebase mode (required for
                                               center-aligned mode) */
    //PWMCON2bits.
    
    //ALTDTR2 = 63;   						/* ALTDeadtime = (65ns / 1.04ns) where 65ns is desired deadtime*/
                                             //  When in center-aligned mode only use ALTDTRx */           
    PHASE2 = PHASE_B_FRE;             				/* In Center-aligned mode the effective period of 
                                               the PWM signal is twice of the value in the 
                                               PHASEx register. So to obtain signal of 400kHz
                                               (or 2.5us period), the PHASE1 register should be
                                               configured for a period of 1.25us.
                                               PHASE1 = ((1.25us) / 1.04ns) = 1202. So effective
                                               period is 2.5us. */
    PDC2 = PHASE_B_FRE * (DUTY_PHASE_B/100.0);                 			/* In Center-aligned mode the effective duty cycle of 
                                               the PWM signal is twice of the value in the 
                                               PDCx register. To achieve 50% duty cycle configure
                                               PDCx register to 0.625us.
                                               PDC1 = ((0.625us) / 1.04ns) = 601. */
    
    /* ~~~~~~~~~~~~~~~~~~~~~~ PWM2 Configuration ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */    
    IOCON3bits.PENH = 1;   					/* PWM1H is controlled by PWM module */
    IOCON3bits.PENL = 1;   					/* PWM1L is controlled by PWM module */
    IOCON3bits.PMOD = 0;   					/* Select Complementary Output PWM mode */
    PWMCON3bits.CAM = 1;                    /* Select Center-aligned mode */
    PWMCON3bits.ITB = 1;                    /* Select Independent timebase mode (required for
                                               center-aligned mode) */
    
     //ALTDTR3 = 63;   						/* ALTDeadtime = (65ns / 1.04ns) where 65ns is desired deadtime*/
                                             /*  When in center-aligned mode only use ALTDTRx */           
    PHASE3 = PHASE_C_FRE;             				/* In Center-aligned mode the effective period of 
                                               the PWM signal is twice of the value in the 
                                               PHASEx register. So to obtain signal of 400kHz
                                               (or 2.5us period), the PHASE1 register should be
                                               configured for a period of 1.25us.
                                               PHASE1 = ((1.25us) / 1.04ns) = 1202. So effective
                                               period is 2.5us. */
    PDC3 = PHASE_C_FRE * (DUTY_PHASE_C/100.0) ;                 			/* In Center-aligned mode the effective duty cycle of 
                                               the PWM signal is twice of the value in the 
                                               PDCx register. To achieve 50% duty cycle configure
                                               PDCx register to 0.625us.
                                               PDC1 = ((0.625us) / 1.04ns) = 601. */

 /* Set Dead Time Values */
/* DTRx Registers are ignored in this mode */
DTR1 = DTR2 = DTR3 = 0;
ALTDTR1 = ALTDTR2 = ALTDTR3 = 25;
/* Configure Faults */
FCLCON1 = FCLCON2 = FCLCON3 = 0x0003;

    /* 1:1 Prescaler */

    PTCON2 = 0x0000;
   PTCON = 0x8000;			   			/* Enable the PWM Module */


}


extern uchar8_t Required_Direction;	// Required mechanical motor direction of 
#define CW	0		// Counter Clock Wise direction
#define CCW	1		// Clock Wise direction
extern uchar8_t Current_Direction;	// Current mechanical motor direction of 
extern uint16_t Phase;
extern signed int PhaseInc; // Delta increments of the Phase variable, calculated 
                     // in the TIMER1 interrupt and used in 
                     // the PWM interrupt (each 50 us)
/* ---------------------- INTERRUPT SERVICE ROUTINES------------------------ */
/* ------------------------------- PWM1 ISR --------------------------------- */
void __attribute__((interrupt, no_auto_psv)) _PWM1Interrupt()
{   
    
 IFS5bits.PWM1IF = 0;	// Clear interrupt flag


	if (Required_Direction == CW)
	{
		if (Current_Direction == CW)
			Phase-= PhaseInc;    // Increment Phase if CW to generate the 
                                  // sinewave only if both directions are equal
		// If Required_Direction is CW (forward) POSITIVE voltage is applied
		#ifdef PHASE_ADVANCE
		SVM(ControlOutput, Phase + PhaseAdvance);	// PhaseAdvance addition
													// produces the sinewave
													// phase shift
		#else
		SVM(ControlOutput, Phase);
		#endif
	}
	else
	{
		if (Current_Direction == CCW)
			Phase += PhaseInc;      // Decrement Phase if CCW to generate
									// the sinewave only if both 
									// directions are equal
		// If Required_Direction is CCW (reverse) NEGATIVE voltage is applied
		#ifdef PHASE_ADVANCE
		SVM(-(ControlOutput+1), Phase + PhaseAdvance);// PhaseAdvance addition
													  // produces the sinewave
													  // phase shift
		#else
		SVM(-(ControlOutput+1), Phase);
		#endif
	}

}


