/****************************************************************************** 
 Section: Included Files
 *****************************************************************************/
#include <p33EV256GM106.h>
#include "types.h"
#include "motorvariables.h"
#include <libq.h>

/*****************************************************************************
 * Section: MACRO DEFINITION'S
 *****************************************************************************/
#define PRESCALERVALUE             (234u)
#define DISABLE                    (0x00u)
#define ONEMINUTE                  (60000u)
#define FIVEHUNMICRO               (35000u)
/* (FCY/pdiv)*60/max rpm * no of pole pairs = 20000000*60/(10000*8)*/
#define MINPERIOD        341u
#define MAXPERIOD        8191u  // 500 rpm //8191u

//2048000 working for 20khz or 500
//65536000 working for 10khz 

#define PHASE_INC_CALC 448000UL  // 2048000   // 2048000 //4096000  //2048000 //4096000 //8192000UL  //(FCY/(PDIV*2*FPWM)*65536)
//#define SPEED_RPM_CALC 3515625UL
#define SPEED_RPM_CALC 4101562UL


#define P_SPEED_PI (0.5)   //prop
#define I_SPEED_PI (0.03)  //integ
#define C_SPEED_PI 0x7FFF                   //windup
#define MAX_SPEED_PI 0x7FFF                 //maxout

typedef struct {
    long    qdSum;          // 1.31 format
    short   qKp;
    short   qKi;
    short   qKc;
    short   qOutMax;
    short   qOutMin;
    short   qInRef; 
    short   qInMeas;
    short   qOut;
    } tPIParm;

void InitPI( tPIParm *pParm);
void CalcPI( tPIParm *pParm);
tPIParm PIparms;
#define SPEED_REF_DIV 16 // speed pot divider
int PeriodFilter;
#define PERIOD_FILTER_CONST 1000 /* the smaller the value, the higher the filter
                                    and the delay introduced */
extern unsigned int MotorStalledCounter; // This variable gets incremented each 
                                      // 1 ms, and is cleared everytime a new
                                      // sector is detected. Used for 
                                      // ForceCommutation and MotorStalled 
                                      // protection functions
/* This is the Timer Interrupt program */

#define _10MILLISEC	 10  // Used as a timeout with no hall effect sensors
                         // transitions and Forcing steps according to the
                         // actual position of the motor
#define _1SEC 1000       // after this time has elapsed, the motor is
                         // consider stalled and it's stopped

uint16_t g_incrementmscounter_u16=0;
extern uint16_t g_markflagcounter_u16;
extern uint16_t g_period_u16;
extern uint16_t g_previous_ic1capturedate_u16;
extern uint16_t g_present_ic1capturedate_u16;
extern signed int PhaseInc; // Delta increments of the Phase variable, calculated 
                            // in the TIMER1 interrupt and used in 
                            // the PWM interrupt (each 50 us)
void Init_Timer1(void);
extern void TogglePort(void);
extern int ControlOutput;
extern uint16_t g_pastcapture_u16;
extern uint16_t g_actualcapture_u16;  
int MeasuredSpeed, RefSpeed = 1000;
void Init_Timer1(void)
{
T1CON=0;
TMR1=0;
PR1=PRESCALERVALUE;
IFS0bits.T1IF=DISABLE;
T1CONbits.TCKPS    = 3; 
IEC0 |=0x0008;
T1CON |=0x8000;
}

void Init_Timer3(void)
{
    PR3 = 0xFFFF;
 IFS0bits.T3IF		= 0;		// Clear Output Compare interrupt flag
 IEC0bits.T3IE		= 1;		// Enable Output Compare interrupts
 T3CONbits.TCKPS    = 3;      
 T3CONbits.TON		= 1;		// Start Timer3 with assumed settings
}


void __attribute__((__interrupt__)) _T1Interrupt(void)
{    
    static int PeriodKFilter = PERIOD_FILTER_CONST;
    static long int PeriodStateVar;
     
      TogglePort();
      g_period_u16 = g_actualcapture_u16 - g_pastcapture_u16;
     //g_period_u16 = g_present_ic1capturedate_u16 - g_previous_ic1capturedate_u16;
     if (g_period_u16 < (unsigned int)MINPERIOD)  // MINPERIOD or 6000 rpm
		g_period_u16 = MINPERIOD;
	 else if (g_period_u16 > (unsigned int)MAXPERIOD) // MAXPERIOD or 60 rpm
		g_period_u16 = MAXPERIOD;
 

  PeriodStateVar+= (((long int)g_period_u16 - (long int)PeriodFilter)*(int)(PeriodKFilter));
  PeriodFilter = (int)(PeriodStateVar>>15);
   
     
  PhaseInc = __builtin_divud((long unsigned int)PHASE_INC_CALC,(unsigned int)PeriodFilter);
  SpeedControl(); 
   
   MotorStalledCounter++;	// We increment a timeout variable to see if the
                            // motor is too slow (not generating hall effect
                            // sensors interrupts frequently enough) or if
                            // the motor is stalled. This variable is cleared
                            // in halls ISRs
    if ((MotorStalledCounter % _10MILLISEC) == 0)
	{
		ForceCommutation();	// Force Commutation if no hall sensor changes 
                            // have occured in specified timeout.
	}
	else if (MotorStalledCounter >= _1SEC)
	{
		//StopMotor(); // Stop motor is no hall changes have occured in 
                     // specified timeout
	}
   
   
IFS0bits.T1IF = 0;
}

void __attribute__((__interrupt__)) _T3Interrupt(void)
{
 IFS0bits.T3IF = 0;   
}
extern unsigned char Required_Direction;
#define CW	0		// Counter Clock Wise direction
#define CCW	1		// Clock Wise direction

void SpeedControl(void)
{
  // ControlOutput = (RefSpeed * 32767.0) / 6000.0 ;  
    double test=0.0;
    //test = (RefSpeed * 5.461);
   //ControlOutput = (uint16_t)(test) ; 
    ControlOutput = 32766;  /* For 1000 rpm*/
 //Required_Direction = CW;    
} 
 
void SpeedControl_Updated(void) {

        PIparms.qKp = P_SPEED_PI;
        PIparms.qKi = I_SPEED_PI;
        PIparms.qKc = C_SPEED_PI;
        PIparms.qOutMax = MAX_SPEED_PI;
        PIparms.qOutMin = - PIparms.qOutMax;
        
        PIparms.qInRef = RefSpeed/SPEED_REF_DIV;
        PIparms.qInMeas = MeasuredSpeed;
        
        
        CalcPI(&PIparms);
        ControlOutput = PIparms.qOut;

        #ifndef CLOSED_LOOP
                ControlOutput = RefSpeed;
        #endif
        // ControlOutput will determine the motor required direction
        if (ControlOutput < 0)
                Required_Direction = CCW;
        else
                Required_Direction = CW;
        return;
}
 


