/******************************************************************************
 *****************************************************************************
 Section: Included Files
 *****************************************************************************
 *****************************************************************************/
#include <xc.h>
#include "Types.h"
#include "CONSTANTS.h"

/*****************************************************************************
 *****************************************************************************
 Section: MACRO DEFINITION'S
 *****************************************************************************
 *****************************************************************************/
#define FIVE_VOLT_CTRL_TRIS _TRISD6
#define FIVE_VOLT_CTRL_PORT PORTDbits.RD6 
#define ENABLE_VOLT_48_CTRL_TRIS _TRISD8
#define ENABLE_VOLT_48_CTRL_PORT _RD8
#define POWER_GOOD_SIGNAL_TRIS  _TRISG8
#define POWER_GOOD_SIGNAL_PORT PORTGbits.RG8
#define MICROCON_ENABLE_SIG_IN_TRIS _TRISB7
#define MICROCON_ENABLE_SIG_IN_PORT PORTBbits.RB7
#define MICROCON_ENABLE_SIG_OUT_TRIS _TRISG9
#define MICROCON_ENABLE_SIG_OUT_PORT _RG9

#define ENABLE_48V_SW_OUT_TRIS _TRISD8
#define ENABLE_48V_SW_OUT_PORT _RD8

#define WD_EN_IN_TRIS  _TRISD5
#define WD_EN_IN_PORT  _RD5

#define COMP_OUT_TRIS _TRISC8
#define COMP_OUT_PORT _RC8

#define WD_SERVICE_SIG_TRIS _TRISC7
#define WD_SERVICE_SIG_PORT _RC7

#define HIGH            (1u)
#define LOW             (0u)
#define INPUT           (1u)
#define OUTPUT          (0u)
#define DIGITAL         (0u)

/*****************************************************************************
  Section:Variables
 *****************************************************************************/
uchar8_t g_power_good_signal_read_u8;
uchar8_t g_microcon_enable_signal_read_u8;

/*****************************************************************************
  function prototype
*****************************************************************************/
void Mode(void);
void H_SO_Enable(void);
void H_SO_Disable(void);
void Sleep_Pin(void);
void Stator_Driver_Enable(void);
void Stator_Driver_Disable(void);
void B_Fault(void);
void Five_Volt_Control(void);
void TwelveV_Buck_Monitor(void);
void MicroControls(void);
void Watchdog_Disable(void);
extern void Field_Enable(void);

uchar8_t g_power_good_signal_read_u8;
uchar8_t g_microcon_enable_signal_read_u8;


/*
 * Pin35 
 * Name: Hbridge_HBL2_Enable
 * AN29/SCK1/RC3
 * Configured as Digital Input
 */

void Hbridge_HBL2_Enable(void)
{
     ANSELCbits.ANSC3 = DIGITAL;
 /* Configure tris as digital */
    _TRISC3 = OUTPUT;
     _RC3=HIGH;
}

/*                         
 Default state for HBH2 is LOW
 Signal: HBH2
 Pin: AN28/SDI1/RA9

 */
void HBridge_HBH2_Enable(void)
{
 ANSELAbits.ANSA9=0; // mode pin is configured as digital pin
 _TRISA9 = OUTPUT;  // mode pin is configured as digital output
 _RA9 = LOW ;
}

/*
 * Pin31
 * RPI24/RA8
 * Function: Active Low signal will enable Hbridge driver
 * Configured as Digital output Value is High 
 */
void HBridge_Enable(void)
{
    _TRISA8 = OUTPUT;
    _RA8 =  LOW;
}


/*
 * Pin31
 * RPI24/RA8
 * Function: Active Low signal will enable Hbridge driver
 * Configured as Digital output Value is High 
 */
void HBridge_Disable(void)
{
    _TRISA8 = OUTPUT;
    _RA8 =  HIGH;
}


/*
 * Pin 64 
 * AN56/RA10
 * Signal: B_En
 * B_Enable --> Stator Driver Enable
 * Configure as Digital output and value is High
 */
void Stator_Driver_Enable(void)
{
 ANSELAbits.ANSA10 = DIGITAL;
 _TRISA10=OUTPUT; /* Initialize output port for B_EN1 disable Pin */ 
 _RA10=LOW;    /* set high for output port B_EN1 disable Pin */ 
}

void Stator_Driver_Disable(void)
{
    ANSELAbits.ANSA10 = DIGITAL;
 _TRISA10=OUTPUT; /* Initialize output port for B_EN1 disable Pin */ 
 _RA10=HIGH;    /* set high for output port B_EN1 disable Pin */ 
}

/*
 Pin54
 * 5v-ctrl 
 * RP70/RD6
 * Configured as digital output value is 1
 */
void Five_Volt_Control(void)
{
 /* Configure tris as digital */
    FIVE_VOLT_CTRL_TRIS = OUTPUT;
    FIVE_VOLT_CTRL_PORT = HIGH;
}

/*
 Pin28
 * FIELD+_VOLT
 * AN13/C3IN2-/U2CTS/FLT6/RE13
 * Configured as digital output value is 1
 */
void TwelveV_Buck_Monitor(void)
{
 /* Configure tris as digital */
    ANSELEbits.ANSE13 = DIGITAL;
    _TRISE13 = OUTPUT;
    _RE13 = HIGH;
}
/*
 Pin42
 * 48V_SW
 * RPI72/RD8
 * Configured as digital output value is 1
 */


void Enable_48V_SW(void)
{
ENABLE_VOLT_48_CTRL_TRIS=OUTPUT;
ENABLE_VOLT_48_CTRL_PORT=HIGH;
}

/*
 Pin53
 * WD_EN_IN
 * RP69/RD5
 * Configured as digital output value is 1
 */
void Watchdog_enable(void)
{
 /* Configure tris as digital*/
 WD_EN_IN_TRIS=OUTPUT;
 WD_EN_IN_PORT=LOW;     
}

void Watchdog_Disable(void)
{
 /* Configure tris as digital*/
 WD_EN_IN_TRIS=OUTPUT;
 WD_EN_IN_PORT=HIGH;     
}
/*
 Pin51
 * WD_SERVICE
 * RN52/RP55/RC7
 * Configured as digital output value is 1
 */
void Watchdog_Service_signal_ON(void)
{
 ANSELCbits.ANSC7=DIGITAL;
 WD_SERVICE_SIG_TRIS=OUTPUT;
 WD_SERVICE_SIG_PORT=HIGH;
}



void Watchdog_Service_signal_OFF(void)
{
 ANSELCbits.ANSC7=DIGITAL;
 WD_SERVICE_SIG_TRIS=OUTPUT;
 WD_SERVICE_SIG_PORT=LOW;
}


void Power_good_signal_read(void)
{
    ANSELGbits.ANSG8=DIGITAL;
    POWER_GOOD_SIGNAL_TRIS=INPUT;
    g_power_good_signal_read_u8=POWER_GOOD_SIGNAL_PORT;   
}

void Microcon_enable_signal_input(void)
{
   ANSELBbits.ANSB7=DIGITAL;
   MICROCON_ENABLE_SIG_IN_TRIS=INPUT;
   g_microcon_enable_signal_read_u8=MICROCON_ENABLE_SIG_IN_PORT; 
}

void Microcon_enable_signal_out(void)
{
    ANSELGbits.ANSG9=DIGITAL;
    MICROCON_ENABLE_SIG_OUT_TRIS=OUTPUT;
    MICROCON_ENABLE_SIG_OUT_PORT=ON;
}


/*
 Pin52
 * COMP_OUT
 * AN51/RP56/RC8
 * Configured as digital output value is 1
 */
void port_toggle(void)
{
    static uint16_t toggle = 0;
   _RC13 = toggle;
    toggle = ~toggle;
    
}
void port_high(void)
{
  //  static uint16 toggle = 0;
  //   static uint16 toggle1 = 0;
 //  ANSELCbits.ANSC8=DIGITAL;
   ANSELCbits.ANSC0=DIGITAL;
   _TRISC0 = OUTPUT;
   _RC0 = 1;
   // COMP_OUT_TRIS=OUTPUT;
   // COMP_OUT_PORT= toggle1 ; 
   // toggle = ~toggle;
  //  toggle1= ~toggle1;
}
   void port_low(void)
{
  //  static uint16 toggle = 0;
  //   static uint16 toggle1 = 0;
 //  ANSELCbits.ANSC8=DIGITAL;
   ANSELCbits.ANSC0=DIGITAL;
   _TRISC0 = OUTPUT;
   _RC0 = 0;
   // COMP_OUT_TRIS=OUTPUT;
   // COMP_OUT_PORT= toggle1 ; 
   // toggle = ~toggle;
  //  toggle1= ~toggle1;
}
void MicroControls(void)
{
	Stator_Driver_Enable();    
	HBridge_Enable();
	HBridge_HBH2_Enable();
	Hbridge_HBL2_Enable();
	//Field_Enable();
	Enable_48V_SW();
	Five_Volt_Control();
	TwelveV_Buck_Monitor();
	Microcon_enable_signal_out();
}
